//=============================================================================
// CharacterObject.cpp
//=============================================================================
#include "Stdinc.h"
#include "CharacterObject.h"
#include "DataManager.h"
#include "MapManager.h"
//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CCharacterObject::CCharacterObject(void)
	: m_nID( INVALID_ID ), m_nDirection ( DIRECTION_D ), m_nState( STATE_STAND_1 ), m_nMoveFrameCnt( 0 ),
	m_nLastUseLeg( STATE_WALK_1 )
{
	RtlZeroMemory( m_nHandle, sizeof(m_nHandle) );
}

//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CCharacterObject::~CCharacterObject(void)
{
}
//-----------------------------------------------------------------------------
// 
//-----------------------------------------------------------------------------
BOOL CCharacterObject::Create( int nID, int nPicID )
{
	m_nID = nID;
	m_nPicID = nPicID;
	m_nDirection = DIRECTION_D;
	m_nState = STATE_STAND_1;

	for ( int iCnt = 0; iCnt < PIC_COUNT; iCnt ++ )
	{
		if ( m_nHandle[iCnt] )
		{
			DeleteGraph( m_nHandle[iCnt] );
			m_nHandle[iCnt] = 0;
		}
	}

	// G
	if ( nPicID == MAP_OBJECT_INVALID_ID )
	{
		m_bIsVisible = FALSE;
		return TRUE;
	}

	return Load( nPicID );
}
//-----------------------------------------------------------------------------
// j
//-----------------------------------------------------------------------------
void CCharacterObject::Delete( void )
{
	for ( int iCnt = 0; iCnt < PIC_COUNT; iCnt ++ )
	{
		if ( m_nHandle[iCnt] )
		{
			DeleteGraph( m_nHandle[iCnt] );
			m_nHandle[iCnt] = 0;
		}
	}
}
//-----------------------------------------------------------------------------
// XV
//-----------------------------------------------------------------------------
void CCharacterObject::FrameMove( void )
{
}
//-----------------------------------------------------------------------------
// `
//-----------------------------------------------------------------------------
void CCharacterObject::Render( void )
{
	if ( !m_bIsVisible )	return;

	int iIndex = m_nDirection * STATE_MAX + m_nState;

	DrawGraphEx( m_nPosX + m_nDrawOffsetX, m_nPosY + m_nDrawOffsetY, m_nHandle[ iIndex ], TRUE ) ;
}
//-----------------------------------------------------------------------------
// [h
//-----------------------------------------------------------------------------
BOOL CCharacterObject::Load( int nPicID )
{
	int nPicIndex = (nPicID - 1) / CHAR_CHIP_MAX_V;
	int nPicPosY  = ((nPicID - 1) % CHAR_CHIP_MAX_V) * ONE_CHAR_HEIGHT;
	int nPicHandle = CDataManager::GetInstance()->m_nHandleCharacter[nPicIndex];

	for ( int iCnt = 0; iCnt < PIC_COUNT; iCnt ++ )
	{
		int nPosX = iCnt * ONE_CHAR_WIDTH;
		m_nHandle[iCnt] = DerivationGraph( nPosX, nPicPosY, ONE_CHAR_WIDTH, ONE_CHAR_HEIGHT, nPicHandle );

	}

	return TRUE;
}
//-----------------------------------------------------------------------------
// SetPosTile
//-----------------------------------------------------------------------------
void CCharacterObject::SetPosTile( int nX, int nY, int nDirection )
{
	m_nPosX = nX * MAP_CHIP_SIZE - (MAP_CHIP_SIZE/2);
	m_nPosY = (nY * MAP_CHIP_SIZE) - ( ONE_CHAR_HEIGHT - MAP_CHIP_SIZE );

	m_nDrawOffsetX = m_nDrawOffsetY = 0;
	m_nDirection = nDirection;
}

//-----------------------------------------------------------------------------
// }bvItZbgʒuɍ킹Aj[V
//-----------------------------------------------------------------------------
int CCharacterObject::CalcState( int nScrollOffset, int nX, int nY )
{
	const int WALK_PATERN[2][4] =
	{
		{
			STATE_STAND_1,		// 
			STATE_WALK_1,		// s1
			STATE_WALK_1,		// s1
			STATE_STAND_1,		// 
		},
		{
			STATE_STAND_1,		// 
			STATE_WALK_2,		// s2
			STATE_WALK_2,		// s2
			STATE_STAND_1,		// 
		},
	};
	int nPaternNo = (nX + nY) % 2;

	int nIndex = (int)(nScrollOffset / 4);

	return WALK_PATERN[nPaternNo][nIndex];
}