//=============================================================================
// ScriptManager.cpp
//=============================================================================
#include "Stdinc.h"
#include "ScriptManager.h"
#include "MapEvent.h"
#include "MapObject.h"
#include "Game.h"
#include "Item.h"

//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CScriptManager::CScriptManager(void)
	: m_pTgtMapObj(NULL), m_LuaState( NULL ), m_LuaCoRoutine( NULL ),
	m_bExecuting( FALSE )
{

}

//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CScriptManager::~CScriptManager(void)
{
	ClearLuaObject();
}

//-----------------------------------------------------------------------------
// Load
//-----------------------------------------------------------------------------
BOOL CScriptManager::Load( int nMapNo, CMapEvent *pMapEvent )
{
	// LUAIuWFNgNA
	ClearLuaObject();

	// XNvgt@C쐬
	char szFileName[MAX_PATH];
	sprintf_s( szFileName, sizeof(szFileName), "%s\\%s%06d.lua", EVENT_DATA_DIR, MAP_EVT_FNAME_PREFIX, nMapNo );

	// Luat@CJēǂݍ
	m_LuaState = luaL_newstate();
	luaL_openlibs( m_LuaState );
	luaopen_base( m_LuaState );

    if ( luaL_dofile( m_LuaState, szFileName )) {
		int iNum = lua_gettop( m_LuaState );

        CSysUtil::Msg( "Script [%s] : %s\n", szFileName, lua_tostring( m_LuaState, iNum ) );
		//LOG_OUTPUT("error : %s\n", lua_tostring( m_LuaState, -1) );
        lua_close( m_LuaState );
		m_LuaState = NULL;

        return FALSE;
    }

	// LUAĂяo֐`
	lua_register( m_LuaState, "expShowMessage", lua_ShowMessage ); 
	lua_register( m_LuaState, "expCloseMessage", lua_CloseMessage ); 
	lua_register( m_LuaState, "expAddMessage", lua_AddMessage ); 
	lua_register( m_LuaState, "expStartFadeout", lua_StartFadeout ); 
	lua_register( m_LuaState, "expStartFadein", lua_StartFadein ); 
	lua_register( m_LuaState, "expShake",		lua_Shake ); 
	lua_register( m_LuaState, "expChangeMap", lua_ChangeMap );
	lua_register( m_LuaState, "expGetTreasureFlag", lua_GetTreasureFlag );
	lua_register( m_LuaState, "expGetTreasure", lua_GetTreasure );
	lua_register( m_LuaState, "expMovePlayer", lua_MovePlayer );
	lua_register( m_LuaState, "expMoveMapObject", lua_MoveMapObject );
	lua_register( m_LuaState, "expWaitFrame", lua_WaitFrame );
	lua_register( m_LuaState, "expOpenStore", lua_OpenStore );
	lua_register( m_LuaState, "expLoadSoundFile", lua_LoadSoundFile );
	lua_register( m_LuaState, "expPlayBGM", lua_PlayBGM );
	lua_register( m_LuaState, "expStopBGM", lua_StopBGM );


	// }bvIuWFNgǂݍ
	for ( int iCnt = 0; iCnt < pMapEvent->m_nWidth * pMapEvent->m_nHeight; iCnt ++ )
	{
		if ( pMapEvent->m_wMapObject[iCnt] != MAP_OBJECT_INVALID_ID )
		{
			int iLocX = iCnt % pMapEvent->m_nWidth;
			int iLocY = iCnt / pMapEvent->m_nWidth;

			CMapObject *pMapObj = new CMapObject();

			// IDݒ
			pMapObj->m_nID = pMapEvent->m_wMapObject[iCnt];

			//  LUA[`R[
			lua_pop( m_LuaState , lua_gettop( m_LuaState ) );	// X^bNNA
			lua_getglobal( m_LuaState, "GetMapObject" );

			// n
			lua_pushnumber( m_LuaState, pMapEvent->m_wMapObject[iCnt] ); // ID

			//CSysDebug::printStack( m_LuaState );	// X^bN_v

			if( lua_pcall( m_LuaState, 1, 1, 0) )
			{
				LOG_OUTPUT("error : %s\n", lua_tostring( m_LuaState, -1) );
				lua_pop( m_LuaState, 1 );
			}
			//CSysDebug::printStack( m_LuaState );	// X^bN_v

			if (lua_type( m_LuaState, 1) ==  LUA_TNIL)
			{	// Yf[^
				delete pMapObj;
				pMapObj = NULL;
				continue;
			}

			// PicID ------------------------------------------------
			lua_getfield( m_LuaState, 1, "PicID");
			//CSysDebug::printStack( m_LuaState );	// X^bN_v
			if (lua_type( m_LuaState, 2) ==  LUA_TNUMBER)
			{
				pMapObj->m_nPicID = (int)lua_tonumber( m_LuaState, 2);
			}
			else
			{
				pMapObj->m_nPicID = MAP_OBJECT_INVALID_ID;
			}
			lua_pop( m_LuaState, 1 );
		

			// Type ----------------------------------------------------
			lua_getfield( m_LuaState, 1, "Type");
			//CSysDebug::printStack( m_LuaState );	// X^bN_v
			if (lua_type( m_LuaState, 2) == LUA_TSTRING)
			{
				pMapObj->m_nType = CMapObject::GetType( lua_tostring( m_LuaState, 2) );
			}
			else
			{
				pMapObj->m_nType = MAP_OBJECT_TYPE_NPC;
			}
			lua_pop( m_LuaState, 1 );

			// ʍstO
			if ( pMapObj->m_nType == MAP_OBJECT_TYPE_FLOOR )
				pMapObj->m_wPassFlag = PASS_FLAG_NONE;
			else
				pMapObj->m_wPassFlag = PASS_FLAG_NO_ENTRY;

			// MoveType ---------------------------------------------------
			lua_getfield( m_LuaState, 1, "MoveType");
			//CSysDebug::printStack( m_LuaState );	// X^bN_v
			if (lua_type( m_LuaState, 2) == LUA_TSTRING)
			{
				pMapObj->m_nMoveType = CMapObject::GetMoveType( lua_tostring( m_LuaState, 2) );
			}
			else
			{
				pMapObj->m_nMoveType = MOVE_FREQ_NORMAL;
			}
			lua_pop( m_LuaState, 1 );

			// MoveSpeed --------------------------------------------------
			lua_getfield( m_LuaState, 1, "MoveSpeed");
			//CSysDebug::printStack( m_LuaState );	// X^bN_v
			if (lua_type( m_LuaState, 2) == LUA_TSTRING)
			{
				pMapObj->m_nMoveSpeed = CMapObject::GetMoveSpeed( lua_tostring( m_LuaState, 2) );
			}
			else
			{
				pMapObj->m_nMoveSpeed = MOVE_SPEED_NORMAL;
			}
			lua_pop( m_LuaState, 1 );

			// MoveFreq ---------------------------------------------------
			lua_getfield( m_LuaState, 1, "MoveFreq");
			//CSysDebug::printStack( m_LuaState );	// X^bN_v
			if (lua_type( m_LuaState, 2) == LUA_TSTRING)
			{
				pMapObj->m_nMoveFreq = CMapObject::GetMoveFreq( lua_tostring( m_LuaState, 2) );
			}
			else
			{
				pMapObj->m_nMoveFreq = MOVE_FREQ_NORMAL;
			}
			lua_pop( m_LuaState, 1 );

			// AniSpeed ---------------------------------------------------
			lua_getfield( m_LuaState, 1, "AniSpeed");
			//CSysDebug::printStack( m_LuaState );	// X^bN_v
			if (lua_type( m_LuaState, 2) == LUA_TSTRING)
			{
				pMapObj->m_nAniSpeed = CMapObject::GetAniSpeed( lua_tostring( m_LuaState, 2) );
			}
			else
			{
				pMapObj->m_nAniSpeed = ANI_SPEED_NORMAL;
			}
			lua_pop( m_LuaState, 1 );

			// ScriptTriger ------------------------------------------------
			lua_getfield( m_LuaState, 1, "ScriptTriger");
			//CSysDebug::printStack( m_LuaState );	// X^bN_v
			if (lua_type( m_LuaState, 2) == LUA_TSTRING)
			{
				pMapObj->m_nScriptTriger = CMapObject::GetScriptTriger( lua_tostring( m_LuaState, 2) );
			}
			else
			{
				pMapObj->m_nScriptTriger = MAP_SCRIPT_TRIGER_NONE;
			}
			lua_pop( m_LuaState, 1 );

			// CxgǗɒǉ
			pMapObj->Create( iLocX, iLocY );
			pMapEvent->m_lstMapObj.push_back( pMapObj );
		}
	}
	// MapIuWFNg̃\[g
	pMapEvent->SortMapObject();


	// Ăяo
	InitProc();

	return TRUE;
}
//-----------------------------------------------------------------------------
// ClearLuaObject
//-----------------------------------------------------------------------------
void CScriptManager::ClearLuaObject( void )
{
	if ( m_LuaState )
	{
//		LOG_OUTPUT( "[%04X]NA\n", m_LuaState );

		lua_pop( m_LuaState , lua_gettop( m_LuaState ));
		lua_close( m_LuaState );
		m_LuaState = NULL;
	}
}

//-----------------------------------------------------------------------------
// InitProc
//-----------------------------------------------------------------------------
BOOL CScriptManager::InitProc( void )
{
	m_pTgtMapObj = NULL;
	m_bExecuting = TRUE;
	CGame::GetInstance()->SetMode( MODE_EVENT );

	//  LUA[`R[
	lua_pop( m_LuaState , lua_gettop( m_LuaState ) );	// X^bNNA

	//LOG_OUTPUT("R[`쐬O [%X]  [%X]\n", m_LuaState, m_LuaCoRoutine );
	// R[`쐬
	m_LuaCoRoutine = lua_newthread( m_LuaState );
	//LOG_OUTPUT("R[`쐬   [%X]  [%X]\n", m_LuaState, m_LuaCoRoutine );

	lua_getglobal( m_LuaCoRoutine, "InitProc");
	//CSysDebug::printStack( m_LuaCoRoutine );	// X^bN_v
    if ( LUA_YIELD != lua_resume(m_LuaCoRoutine, 0)  )
	{	// ͖
		m_bExecuting = FALSE;
		CGame::GetInstance()->SetToPrevMode();
    }
	//CSysDebug::printStack( m_LuaCoRoutine );	// X^bN_v

	return TRUE;
}
//-----------------------------------------------------------------------------
// InitProc
//-----------------------------------------------------------------------------
BOOL CScriptManager::BattleFinishedProc( void )
{
	m_pTgtMapObj = NULL;
	m_bExecuting = TRUE;
	CGame::GetInstance()->SetMode( MODE_EVENT );

	//  LUA[`R[
	lua_pop( m_LuaState , lua_gettop( m_LuaState ) );	// X^bNNA

	//LOG_OUTPUT("R[`쐬O [%X]  [%X]\n", m_LuaState, m_LuaCoRoutine );
	// R[`쐬
	m_LuaCoRoutine = lua_newthread( m_LuaState );
	//LOG_OUTPUT("R[`쐬   [%X]  [%X]\n", m_LuaState, m_LuaCoRoutine );

	lua_getglobal( m_LuaCoRoutine, "BattleFinishedProc");
	//CSysDebug::printStack( m_LuaCoRoutine );	// X^bN_v
    if ( LUA_YIELD != lua_resume(m_LuaCoRoutine, 0)  )
	{	// ͖
		m_bExecuting = FALSE;
		CGame::GetInstance()->SetToPrevMode();
    }
	//CSysDebug::printStack( m_LuaCoRoutine );	// X^bN_v

	return TRUE;
}
//-----------------------------------------------------------------------------
// Start
//-----------------------------------------------------------------------------
BOOL CScriptManager::Start( CMapObject *pMapObj )
{
	m_pTgtMapObj = pMapObj;
	m_bExecuting = TRUE;

	//  LUA[`R[
	lua_pop( m_LuaState , lua_gettop( m_LuaState ) );	// X^bNNA

	//LOG_OUTPUT("R[`쐬O [%X]  [%X]\n", m_LuaState, m_LuaCoRoutine );
	// R[`쐬
	m_LuaCoRoutine = lua_newthread( m_LuaState );
	//LOG_OUTPUT("R[`쐬   [%X]  [%X]\n", m_LuaState, m_LuaCoRoutine );

	lua_getglobal( m_LuaCoRoutine, "ExecObject");
	lua_pushnumber( m_LuaCoRoutine, pMapObj->m_nID );
	//CSysDebug::printStack( m_LuaCoRoutine );	// X^bN_v
    if ( LUA_YIELD != lua_resume(m_LuaCoRoutine, 1)  )
	{	// ͖
		m_bExecuting = FALSE;
		CGame::GetInstance()->SetToPrevMode();
    }
	//CSysDebug::printStack( m_LuaCoRoutine );	// X^bN_v

	return TRUE;
}
//-----------------------------------------------------------------------------
// ExecNext
//-----------------------------------------------------------------------------
BOOL CScriptManager::ExecNext( void )
{
	LOG_OUTPUT("CScriptManager::ExecNext\n");
	//  LUA[`R[
	//CSysDebug::printStack( m_LuaCoRoutine );	// X^bN_v
    if ( LUA_YIELD != lua_resume(m_LuaCoRoutine, 1)  )
	{	// ͖
		m_bExecuting = FALSE;
		CGame::GetInstance()->SetToPrevMode();
    }
	//CSysDebug::printStack( m_LuaCoRoutine );	// X^bN_v

	return TRUE;
}

// ============================================================================
//  LuaR[obN
// ============================================================================
//-----------------------------------------------------------------------------
// lua_ShowMessage
//-----------------------------------------------------------------------------
int lua_ShowMessage(lua_State *L)
{
	LOG_OUTPUT("lua_ShowMessage\n");
	//CSysDebug::printStack( L );	// X^bN_v

	// 擾
	const char *lpzMessage = lua_tostring( L, 1 );

	CGame::GetInstance()->ShowMessage( lpzMessage );

	return 0;
}
//-----------------------------------------------------------------------------
// lua_CloseMessage
//-----------------------------------------------------------------------------
int lua_CloseMessage( lua_State *L )
{
	LOG_OUTPUT("lua_CloseMessage\n");
	//CSysDebug::printStack( L );	// X^bN_v

	CGame::GetInstance()->CloseMessage();

	return 0;
}
//-----------------------------------------------------------------------------
// lua_AddMessage
//-----------------------------------------------------------------------------
int lua_AddMessage( lua_State *L )
{
	LOG_OUTPUT("lua_AddMessage\n");
	//CSysDebug::printStack( L );	// X^bN_v

	// 擾
	const char *lpzMessage = lua_tostring( L, 1 );
	CGame::GetInstance()->AddMessage( lpzMessage );

	return 0;
}
//-----------------------------------------------------------------------------
// lua_StartFadeout
//-----------------------------------------------------------------------------
int lua_StartFadeout( lua_State *L )
{
	LOG_OUTPUT("lua_StartFadeout\n");
	//CSysDebug::printStack( L );	// X^bN_v

	int nR = 255;
	int nG = 255;
	int nB = 255;
	UINT nFrame = 30;
	DWORD dwColor;

	// p^擾
	nFrame = (int)lua_tonumber( L, 4);
	nB = (int)lua_tonumber( L, 3);
	nG = (int)lua_tonumber( L, 2);
	nR = (int)lua_tonumber( L, 1);

	//CSysDebug::printStack( L );	// X^bN_v
	dwColor = GetColor( nR, nG, nB );

	CGame::GetInstance()->StartFadeout( dwColor, nFrame );

	return 0;
}
//-----------------------------------------------------------------------------
// lua_StartFadein
//-----------------------------------------------------------------------------
int lua_StartFadein( lua_State *L )
{
	LOG_OUTPUT("lua_StartFadein\n");
	//CSysDebug::printStack( L );	// X^bN_v

	int nR = 255;
	int nG = 255;
	int nB = 255;
	UINT nFrame = 30;
	DWORD dwColor;

	// p^擾
	nFrame = (int)lua_tonumber( L, 4);
	nB = (int)lua_tonumber( L, 3);
	nG = (int)lua_tonumber( L, 2);
	nR = (int)lua_tonumber( L, 1);

	//CSysDebug::printStack( L );	// X^bN_v
	dwColor = GetColor( nR, nG, nB );

	CGame::GetInstance()->StartFadein( dwColor, nFrame );

	return 0;
}
//-----------------------------------------------------------------------------
// lua_Shake
//-----------------------------------------------------------------------------
int lua_Shake( lua_State *L )
{
	LOG_OUTPUT("lua_Shake\n");
	//CSysDebug::printStack( L );	// X^bN_v
	double dblSpeed;
	int nType, nFrame, nScale;
	nScale   = (int)lua_tonumber( L, 4);
	dblSpeed = (double)lua_tonumber( L, 3);
	nFrame   = (int)lua_tonumber( L, 2);
	nType    = (int)lua_tonumber( L, 1);

	//CSysDebug::printStack( L );	// X^bN_v

	CGame::GetInstance()->StartShake( nType, nFrame, dblSpeed, nScale );

	return 0;
}
//-----------------------------------------------------------------------------
// lua_ChangeMap
//-----------------------------------------------------------------------------
int lua_ChangeMap( lua_State *L )
{
	LOG_OUTPUT("lua_ChangeMap\n");
	//CSysDebug::printStack( L );	// X^bN_v

	int nMapID, nLocX, nLocY, nDir;

	nDir   = (int)lua_tonumber( L, 4);
	nLocY  = (int)lua_tonumber( L, 3);
	nLocX  = (int)lua_tonumber( L, 2);
	nMapID = (int)lua_tonumber( L, 1);

	CGame::GetInstance()->ChangeMapRequest( nMapID, nLocX, nLocY, nDir );

	return 0;
}
//-----------------------------------------------------------------------------
// lua_GetTreasureFlag
//-----------------------------------------------------------------------------
int lua_GetTreasureFlag( lua_State *L )
{
	LOG_OUTPUT("lua_GetTreasureFlag\n");
	//CSysDebug::printStack( L );	// X^bN_v

	int nMapID, nEventID;

	nEventID = (int)lua_tonumber( L, 2);
	nMapID	 = (int)lua_tonumber( L, 1);
	lua_pop( L , lua_gettop( L ) );	// X^bNNA

	int nIndex;
	int nRet = CGame::GetInstance()->GetTreasureFlag( nMapID, nEventID, &nIndex );

	lua_pushnumber( L, nRet ); // n

	//CSysDebug::printStack( L );	// X^bN_v

	return 1;
}
//-----------------------------------------------------------------------------
// lua_GetTreasure
//-----------------------------------------------------------------------------
int lua_GetTreasure( lua_State *L )
{
	LOG_OUTPUT("lua_GetTreasure\n");
	//CSysDebug::printStack( L );	// X^bN_v

	int nMapID, nEventID;

	nEventID = (int)lua_tonumber( L, 2 );
	nMapID	 = (int)lua_tonumber( L, 1 );
	lua_pop( L , lua_gettop( L ) );	// X^bNNA

	CItem *pItem;
	CGame::GetInstance()->GetTreasure( nMapID, nEventID, &pItem );
	lua_pushstring( L, pItem->szName ); // n
	LOG_OUTPUT("[%s]\n", pItem->szName);

	//CSysDebug::printStack( L );	// X^bN_v
	delete pItem;

	return 1;
}
//-----------------------------------------------------------------------------
// lua_MovePlayer
//-----------------------------------------------------------------------------
int lua_MovePlayer( lua_State *L )
{
	LOG_OUTPUT("lua_MovePlayer\n");
	//CSysDebug::printStack( L );	// X^bN_v

	int nDirection, nSpeed;

	nSpeed		= (int)lua_tonumber( L, 2 );
	nDirection	= (int)lua_tonumber( L, 1 );
	lua_pop( L , lua_gettop( L ) );	// X^bNNA

	CGame::GetInstance()->MovePlayer( nDirection, nSpeed );

	//CSysDebug::printStack( L );	// X^bN_v

	return 0;
}
//-----------------------------------------------------------------------------
// lua_MoveMapObject
//-----------------------------------------------------------------------------
int lua_MoveMapObject( lua_State *L )
{
	LOG_OUTPUT("lua_MoveCharacter\n");
	//CSysDebug::printStack( L );	// X^bN_v

	int nID, nDirection, nSpeed;

	nSpeed		= (int)lua_tonumber( L, 3 );
	nDirection	= (int)lua_tonumber( L, 2 );
	nID			= (int)lua_tonumber( L, 1 );
	lua_pop( L , lua_gettop( L ) );	// X^bNNA

	//LOG_OUTPUT("ID[%d] Dir[%d] Spd[%d]\n", nID, nDirection, nSpeed );

	CGame::GetInstance()->MoveMapObject( nID, nDirection, nSpeed );

	return 0;
}
//-----------------------------------------------------------------------------
// lua_WaitFrame
//-----------------------------------------------------------------------------
int lua_WaitFrame( lua_State *L )
{
	LOG_OUTPUT("lua_WaitFrame\n");
	//CSysDebug::printStack( L );	// X^bN_v

	int nFrame;

	nFrame	= (int)lua_tonumber( L, 1 );
	lua_pop( L , lua_gettop( L ) );	// X^bNNA

	CGame::GetInstance()->WaitFrame( nFrame );

	return 0;
}
//-----------------------------------------------------------------------------
// lua_OpenStore
//-----------------------------------------------------------------------------
int lua_OpenStore( lua_State *L )
{
	LOG_OUTPUT("lua_OpenStore\n");
	//CSysDebug::printStack( L );	// X^bN_v

	int nType, nMsgPtn, nRate;	
	nRate	= (int)lua_tonumber( L, 3 );
	nMsgPtn	= (int)lua_tonumber( L, 2 );
	nType	= (int)lua_tonumber( L, 1 );

	// ie[u
	int iItemCount = (int)lua_objlen( L, 4 );
	int *lstItem = new int [iItemCount];
	for ( int iCnt = 0; iCnt < iItemCount; iCnt ++ )
	{
		lua_pushnumber( L, iCnt+1 );
		lua_gettable( L, 4 );
		lstItem[iCnt] = (int)lua_tonumber( L, 5 + iCnt );

		LOG_OUTPUT("ItemID [%d]\n", lstItem[iCnt]);
	}

	lua_pop( L , lua_gettop( L ) );	// X^bNNA

	CGame::GetInstance()->OpenStore( nType, nMsgPtn, nRate, iItemCount, lstItem );

	delete [] lstItem;

	return 0;
}

//-----------------------------------------------------------------------------
// lua_LoadSoundFile
//-----------------------------------------------------------------------------
int lua_LoadSoundFile( lua_State *L )
{
	LOG_OUTPUT("lua_LoadSoundFile\n");
	//CSysDebug::printStack( L );	// X^bN_v

	int nID;
	nID	= (int)lua_tonumber( L, 1 );
	lua_pop( L , lua_gettop( L ) );	// X^bNNA

	CGame::GetInstance()->LoadSoundFile( nID );

	return 0;
}
//-----------------------------------------------------------------------------
// lua_PlayBGM
//-----------------------------------------------------------------------------
int lua_PlayBGM( lua_State *L )
{
	LOG_OUTPUT("lua_PlayBGM\n");
	CSysDebug::printStack( L );	// X^bN_v

	int nID;
	BOOL bTopPosFlag;
	bTopPosFlag = (int)lua_tonumber( L, 2 );
	nID			= (int)lua_tonumber( L, 1 );
	lua_pop( L , lua_gettop( L ) );	// X^bNNA

	CGame::GetInstance()->PlayBGM( nID, bTopPosFlag );

	return 0;
}
//-----------------------------------------------------------------------------
// lua_StopBGM
//-----------------------------------------------------------------------------
int lua_StopBGM( lua_State *L )
{
	CGame::GetInstance()->StopBGM( );
	return 0;
}
